function[compPowStruct] = ps2_ecog_subjPow_doComp(uSubj,eName,compar,powConfigNum,fRange)
%This function performs a power comparison between two trial types for a
%particular freq. band

%Inputs
%uSubj .....   subject id
%eName .....   electrode name
%comparison ... see list below
%powConfig ...  which power files to load (default = 1)
%fRange ...  frequency range of power to analyze (default = [70 200])

%Outputs
%compStruct ... structure which contains power information for condition 1,
                %condition 2, difference score, and cluster-based statistics

% parse inputs
if ~exist('powConfigNum','var') || isempty(powConfigNum)
    powConfigNum = 1;
end
if ~exist('fRange','var') || isempty(fRange)
    fRange = [70 200];
end

% check if saved
clusConfig = clusConfig_local();
saveDir = fullfile(clusConfig.saveDir,compar,['powConfig' num2str(powConfigNum) '-' num2str(fRange(1)) '-' num2str(fRange(2))]);
saveFile = [uSubj '-' eName '.mat'];
cd_mkdir(saveDir);
if exist(saveFile,'file')
    load(saveFile)
else
    % get events
    [fbEvents,choiceEvents,stimEvents] = ps2_ecog_subjPow_getSubjEv(uSubj);

    % get power
    [fbPow,chPow,stPow,ret_fb,ret_ch,powConfig] = ...
        ps2_ecog_subjPow_getSubjPow(uSubj,eName,powConfigNum);
    
    % select sub-group of events for this elec
    fbEvents = fbEvents(ret_fb);
    stimEvents = stimEvents(ret_ch);
    choiceEvents = choiceEvents(ret_ch);
    
    % collapse within freq range
    [fbPow,chPow,stPow] = getBandPow_local(fbPow,chPow,stPow,powConfig,fRange);

    % get ret lbls - identifies appropriate labels to use below
    [retLbls,diffMat,diffLbls,filtLbl,modelLbl] = getLbl_local(compar);
    
    % filt lbl - creates masks based on labels that were just created
    if ~isempty(filtLbl)
        [filt_fb,filt_ch,~,~,~,~,~,fb_pair_lbl_unfilt] = ps2_ecog_subjPow_filtSubjEv(uSubj,filtLbl);
        filt_fb_ret = filt_fb(ret_fb);
        filt_ch_ret = filt_ch(ret_ch);
        %fb_pair_lbl = fb_pair_lbl_unfilt(ret_fb);
        
        fbEvents = fbEvents(filt_fb_ret);
        stimEvents = stimEvents(filt_ch_ret);
        choiceEvents = choiceEvents(filt_ch_ret);
        fb_pair_lbl = fb_pair_lbl_unfilt(filt_fb_ret);
        pair_list = unique(fb_pair_lbl);

        fbPow = fbPow(filt_fb_ret,:);
        chPow = chPow(filt_ch_ret,:);
        stPow = stPow(filt_ch_ret,:);
    end
    if ~isempty(modelLbl)
        % get model
        [modelStats,paramStruct] =  ps2_ecog_subjPow_rlModel(uSubj,filtLbl,modelLbl);
    else 
        modelStats = []; paramStruct = [];
    end
    
    %filter model stats
    if ~isempty(modelStats)
        names = fieldnames(modelStats);
        for n = 1:length(names)
            thisVec = modelStats.(names{n});
            if ~any(strcmp(names{n},{'LLE','pseudRSq','sumSq','active_rule','rul_a','rul_b'}))
                %modelStats.(names{n}) = ...
                 %  thisVec(filt_fb(ret_fb));
                modelStats.(names{n}) = ...
                   thisVec(ret_fb(filt_fb));
            else
               modelStats.(names{n}) = ...
                   thisVec; 
            end
        end
    end  
       
    % saves power vectors
    compPowStruct = struct;
    for r = 1:length(retLbls)
        if ~isempty(fbEvents)
            [powVec] = filtEvPow_local(retLbls{r},fbEvents,...
            choiceEvents,stimEvents,fbPow,chPow,stPow,modelStats,paramStruct);
        else
            powVec = nan(2,length(powConfig.timeWins_lbl));
        end
        compPowStruct.(retLbls{r}) = powVec;
    end

    % do cluster-based comparison
    for i = 1:size(diffMat,1)
        [compPowStruct.(diffLbls{i})] = ...
            doComparison_local(compPowStruct.(retLbls{diffMat(i,1)}),compPowStruct.(retLbls{diffMat(i,2)}),powConfig);
    end
    
    % get beh stats
    if ~isempty(fbEvents)&&~isempty(filtLbl)
        [behStats] = behStats_local(uSubj,fbEvents,fb_pair_lbl,pair_list);
    else
        behStats = [];
    end
    
    % add behavioral stuff
    compPowStruct.modelStats = modelStats;
    compPowStruct.paramStruct = paramStruct;
    compPowStruct.fbEvents = fbEvents;
    compPowStruct.behStats = behStats;
    
    %save 
    cd(saveDir)
    save(saveFile,'compPowStruct','clusConfig')
end

% FUNCTIONS %
function[fbPow,chPow,stPow] = getBandPow_local(fbPow,chPow,stPow,powConfig,fRange);
[~,fStart] = min(abs(fRange(1)-powConfig.freQ));
[~,fEnd] = min(abs(fRange(2)-powConfig.freQ));
fbPow = shiftdim(nanmean(fbPow(fStart:fEnd,:,:),1))';
chPow = shiftdim(nanmean(chPow(fStart:fEnd,:,:),1))';
stPow = shiftdim(nanmean(stPow(fStart:fEnd,:,:),1))';

function [retLbls,diffMat,diffLbls,filtLbl,modelLbl] = getLbl_local(compar)
% defaults
diffMat = []; % each row in this matrix marks a new comparison
diffLbls = {};
filtLbl = '';
modelLbl = '';
switch compar
    case 'fb_posNeg'
        retLbls = {'fb_pos','fb_neg'};
        diffMat = [1 2]; % each row in this matrix marks a new comparison
        diffLbls = {'posVneg'};
    case 'fb_pairWise'
        retLbls = {'fb_pos_AB','fb_pos_CD','fb_pos_EF','fb_neg_AB','fb_neg_CD','fb_neg_EF'};
    case 'fb_RPE_LRN'
        retLbls = {'fb_pos','fb_neg','fb_pos_loExp','fb_pos_medExp','fb_pos_hiExp',...
            'fb_neg_loExp','fb_neg_medExp','fb_neg_hiExp'};
        filtLbl = 'LRN';
        modelLbl = 'qPosBeta';
    case 'fb_RPE_train'
        retLbls = {'fb_pos','fb_neg','fb_pos_loExp','fb_pos_medExp','fb_pos_hiExp',...
            'fb_neg_loExp','fb_neg_medExp','fb_neg_hiExp'};
        filtLbl = 'train';
        modelLbl = 'qPosBeta';
    case 'fb_LR'
        retLbls = {'fb_L','fb_R'};
        diffMat = [1 2]; % each row in this matrix marks a new comparison
        diffLbls = {'leftVRight'};
    case 'fb_earlyLate'; %trials 1-10 vs. trials 30 and above (see selectivity analysis0
        retLbls = {'fb_pos','fb_neg','fb_pos_early','fb_pos_late','fb_neg_early','fb_neg_late'};
end

function[powVec] = filtEvPow_local(retLbl,fbEvents,...
    choiceEvents,stimEvents,fbPow,chPow,stPow,modelStats,paramStruct)
ev = fbEvents;
switch retLbl
    case 'fb_pos' 
        retain = [ev.fb]==1;
    case 'fb_neg' 
        retain = [ev.fb]==0;
    case 'fb_pos_AB' 
        retain = [ev.fb]==1&[ev.pairNum]==0;
    case 'fb_pos_CD' 
        retain = [ev.fb]==1&[ev.pairNum]==1;
    case 'fb_pos_EF' 
        retain = [ev.fb]==1&[ev.pairNum]==2;
    case 'fb_neg_AB' 
        retain = [ev.fb]==0&[ev.pairNum]==0;
    case 'fb_neg_CD' 
        retain = [ev.fb]==0&[ev.pairNum]==1;
    case 'fb_neg_EF' 
        retain = [ev.fb]==0&[ev.pairNum]==2;
    case 'fb_pos_loExp'
        retain = [ev.fb]==1 &...
            modelStats.exp_choice<=prctile(modelStats.exp_choice,33);
    case 'fb_pos_medExp'
        retain = [ev.fb]==1 &...
             modelStats.exp_choice>prctile(modelStats.exp_choice,33)&...
             modelStats.exp_choice<prctile(modelStats.exp_choice,66);
    case 'fb_pos_hiExp'
        retain = [ev.fb]==1 &...
            modelStats.exp_choice>prctile(modelStats.exp_choice,66);
    case 'fb_neg_loExp'
        retain = [ev.fb]==0 &...
            modelStats.exp_choice<=prctile(modelStats.exp_choice,33);
    case 'fb_neg_medExp'
        retain = [ev.fb]==0 &...
             modelStats.exp_choice>prctile(modelStats.exp_choice,33)&...
             modelStats.exp_choice<prctile(modelStats.exp_choice,66);
    case 'fb_neg_hiExp'
        retain = [ev.fb]==0 &...
            modelStats.exp_choice>prctile(modelStats.exp_choice,66);
    case 'fb_L'
        retain = [ev.button] == 1;
    case 'fb_R'
        retain = [ev.button] == 2;
    case 'fb_pos_early'
        retain = [ev.fb]==1&[ev.tnum]<=10;
    case 'fb_neg_early'
        retain = [ev.fb]==0&[ev.tnum]<=10;
    case 'fb_pos_late'
        retain = [ev.fb]==1&[ev.tnum]>=30;
    case 'fb_neg_late'
        retain = [ev.fb]==0&[ev.tnum]>=30;        
end
powVec = fbPow(retain,:);

function [truStats] = doComparison_local(powVec1,powVec2,powConfig);

% get clus alpha values
clusConfig = clusConfig_local;

[truStats.tVec,truStats.pVec,truStats.df,...
    truStats.posClusStats,truStats.negClusStats,...
    truStats.posClusInd,truStats.negClusInd] = ...
    clusStats_wrapper_local(powVec1,powVec2);
   
% shufled power
shufPow = [powVec1;powVec2];

for i = 1:clusConfig.iters
    %shuf fb events
    shufInd = logical(randi([0 1],[1 size(shufPow,1)]));
    
    [~,~,~,shuf_posClusStats,shuf_negClusStats] = ...
    clusStats_wrapper_local(shufPow(shufInd,:),shufPow(~shufInd,:));
    
    shufStats(i).posClusStats = shuf_posClusStats(1);
    shufStats(i).negClusStats = shuf_negClusStats(1);
    
    if mod(i,100) == 0
      disp(i)
    end
end

% calc p-values and adjust indices

truStats.posClusStatsP = [];truStats.negClusStatsP = [];
for t = 1:length(truStats.posClusStats)
    [truStats.posClusStatsP(t)] = calcP(truStats.posClusStats(t),[shufStats.posClusStats]);
    [truStats.posClusInd{t}] = powConfig.timeWins_lbl(truStats.posClusInd{t});
end
for t = 1:length(truStats.negClusStats)
    [truStats.negClusStatsP(t)] = calcP(truStats.negClusStats(t),[shufStats.negClusStats]);
    [truStats.negClusInd{t}] =  powConfig.timeWins_lbl(truStats.negClusInd{t});
end



function [tVec,pVec,df,...
    posClusStats,negClusStats,posClusInd,negClusInd] = clusStats_wrapper_local(powVec1,powVec2)
% get clus alpha values
clusConfig = clusConfig_local;

% do one-tailed t-test
[~,pVec,~,stats] = ttest2(powVec1,powVec2,[],'right',[],1);
tVec = stats.tstat;
df = stats.df;

%calc clus stat
[posClusStats, negClusStats, posClusInd, negClusInd] = ...
    calcClusStat(tVec,pVec,clusConfig.alfa);



function[posClusStats, negClusStats, posClusInd, negClusInd] = calcClusStat(tMat,pMat,alfa)
%This function identifies contiguous tf clusters. 
%Inputs:
%tMat: tf matrix of t values
%pMat = tf matrix of p values (one-tailed)
%alfaVec = [0.025 0.975]; one-tailed p-thresholds; p<0.025  will identify
           %positive tf activations,  p>0.975 will reveal negative tf activations
%Outputs:
%posClusStats .... vector of positive activations ordered by magniture
%negClusStats ..... vector of negative activations ordered by magnitude
%posClusInd ...... cell array of sig. indices for positive activations
%negClusInd ..... cell array of sig. indices for negative activaions 

%Positive activation
%identify sig. windows (e.g., p < 0.05)
inds = find(pMat<alfa(1));
[posClusStats,posClusInd] = doClusCalc(inds,tMat,pMat);

inds = find(pMat>alfa(2));
[negClusStats,negClusInd] = doClusCalc(inds,tMat,pMat);

function[clusStats,clusInds] = doClusCalc(inds,tMat,pMat)
if length(inds)<2 % if 
    clusStats = [0];
    clusInds = {[]};
    return
end
%identify contiguous windows
[subs(:,1), subs(:,2)] = ind2sub(size(pMat),inds); %this step will be helpful for spectral analyses
z = linkage(subs);
clusObs=cluster(z,'cutoff',sqrt(2),'criterion','distance');

%calc time of onset and cluster stats for each ''activation''
clusStats = nan(1,length(unique(clusObs)));
clusInds = cell(1,length(unique(clusObs)));
for i = 1:length(unique(clusObs))
    clusStats(i) = sum(tMat(inds(clusObs==i)));
    clusInds{i} = inds(clusObs==i);
end

%sort by magnitude
[~,magInd] = sort(abs(clusStats),'descend');
clusStats = clusStats(magInd);
clusInds = clusInds(magInd);


function [pVal] = calcP(true_t,shuf_t)
%For t-stats, it is a one-tailed p value; (high t gives low p, low t gives
%high p) 
%For Cluster t-stats, it is a two-tailed p-value for pos. clusters and neg
%clusters, low ps are sig
%For f-stats it is a two-tailed pvalue; low ps are sig.
numDim = sum(size(true_t)>1);
if numDim == 0 %a single distribution of t-values (e.g clus stats)
    shuf_t = squeeze(shuf_t);
    if true_t >= 0 % for pos cluster stats
        pVal = sum(true_t<=shuf_t)./length(shuf_t);
    elseif true_t < 0 % for neg cluster stats
        pVal = sum(true_t>=shuf_t)./length(shuf_t);
    end
else
    if numDim == 1
       tMat = repmat(true_t,[1 size(shuf_t,2) size(shuf_t,3)]);
    elseif numDim == 2
       tMat = repmat(true_t,[1 1 size(shuf_t,3)]);
    end
    %get a single null value at each freq 
    shuf_t = repmat(nanmean(shuf_t,2),[1 size(tMat,2) 1]);
    pVal = sum(tMat<=shuf_t,3)./size(shuf_t,3);
end


function[clusConfig] = clusConfig_local();
dirs = ps2_dirs;
clusConfig.alfa = [0.025 0.975];
clusConfig.iters = 1000;
clusConfig.saveDir = fullfile(dirs.analyses,'ecog','subjPow','clusStats');

function[behStats] = behStats_local(uSubj,events,fb_pair_lbl,pair_list)
correct = [events.correct];
button = [events.button];
for p = 1:length(pair_list)
    pairIdx = strcmp(pair_list{p},fb_pair_lbl);
    
    % early idx
    early_idx = false(size(pairIdx));
    early_idx(find(pairIdx,10,'first')) = true;
  
    fb = [events.fb]==1; %reward trials
    fb_lag1 = circshift(fb,[0 1]); %trials after wins (1) or losses (0)
    samePair_lag1 = [0 (diff(pairIdx))==0]; %trials where pair was same as prev trial
    sameItem_lag1 = [0 (diff(correct))==0]; %trials where the item choice was same as prev trial
    sameButton_lag1 = [0 (diff(button))==0]; %trials where the button choice was same as prev trial

    % pair number 
    thisStats(p).pairNum = unique([events(pairIdx).pairNum]);
    
    %num trials
    thisStats(p).numTrials = sum(pairIdx);
    
    %pair-wise acc
    thisStats(p).acc = mean(correct(pairIdx));
     
    % acc_01 (accuracy on first 10 trials)
    thisStats(p).acc01 = mean(correct(find(pairIdx,10,'first'))); 
    
    % acc_10 (accuracy on last 10 trials)
    thisStats(p).acc10 = mean(correct(find(pairIdx,10,'last'))); 

    % win-lag1: behavior on (AB) trials following reward where the same pair
    % was available. Only during the first 10 trials
    retain = pairIdx&fb_lag1&samePair_lag1&early_idx;
    thisStats(p).winStay = mean(sameItem_lag1(retain)); %win-same item
    thisStats(p).winSameButton = mean(sameButton_lag1(retain)); %win-same button
    thisStats(p).winStayZ = mean(sameItem_lag1(retain)) - mean(sameItem_lag1(pairIdx&~fb_lag1&samePair_lag1)); %win-same item - lose same item
    
    % lose-lag1: behavior on (AB) trials following loss trials where the same pair 
    % was available. Only during the first 10 trials
    retain = pairIdx&~fb_lag1&samePair_lag1&early_idx;
    thisStats(p).loseSwitch = mean(~sameItem_lag1(retain)); %lose-switch
    thisStats(p).loseSwitchButton = mean(~sameButton_lag1(retain)); %win-switch button
end
behStats = struct;
behStats.uSubj = uSubj;
names = fieldnames(thisStats); 
for n = 1:length(names)
   behStats.(names{n}) = nanmean([thisStats.(names{n})]); 
end

% for plotting
behStats.correct = [events.correct];
behStats.pairNum = [events.pairNum];
behStats.fb = [events.fb];
% % by pair
% pairNum = [thisStats.pairNum];
% pairLbl = {'AB','CD','EF'};
% for p = 1:length(pairLbl)
%     behStats.(['acc10_' pairLbl{p}]) = nanmean([thisStats(pairNum==p-1).acc10]);
%     behStats.(['acc_' pairLbl{p}]) = nanmean([thisStats(pairNum==p-1).acc]);
%     behStats.(['winStay_' pairLbl{p}]) = nanmean([thisStats(pairNum==p-1).winStay]);
%     behStats.(['winSameButton_' pairLbl{p}]) = nanmean([thisStats(pairNum==p-1).winSameButton]);
%     behStats.(['loseSwitch_' pairLbl{p}]) = nanmean([thisStats(pairNum==p-1).loseSwitch]);
%     behStats.(['loseSwitchButton_' pairLbl{p}]) = nanmean([thisStats(pairNum==p-1).loseSwitchButton]);
% end